<?php
/*
Plugin Name: EPP - Edit Post with Passwords Basic
Description: Allow guest users to edit posts from the frontend with a unique edit password.
Version: 1.0
Author: Tahsinur Tamim
Author URI: https://profiles.wordpress.org/tamim34/
License: GPL v2 or later
License URI: http://www.gnu.org/licenses/gpl-2.0.txt
*/

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

define('EPPB_PREFIX', 'eppb_');

// Add a settings page
function eppb_menu_item() {
    add_menu_page(
        'Edit Post with Password Settings',
        'Edit Post with Password',
        'manage_options',
        'edit-post-password-settings',
        'eppb_render_settings_page'
    );
}
add_action('admin_menu', 'eppb_menu_item');

function eppb_render_settings_page() {
    ?>
    <div class="wrap">
        <h2>Edit Post with Password Settings</h2>
        <div>
        <hr>    
        <label for="post_type">Select Post Type:</label>
            <select name="post_type" id="post_type">
                <option selected>Post</option>
                <option disabled="true">Custom Post Type (PRO)</option>
            </select>
            <hr>   
            <h3>Our Pro Features</h3>
            <ol>
                <li>
                    <p><strong>Settings Page:</strong></p>
                    <ul>
                        <li>Define post types eligible for frontend editing through a user-friendly settings page.</li>
                        <li>Easily configure allowed <em><strong>post types</strong></em> according to your preferences.</li>
                    </ul>
                </li>
                <li>
                    <p><strong>Edit Password:</strong></p>
                    <ul>
                        <li><strong>Generate and validate a unique</strong> edit password for each post.</li>
                        <li><strong>Enhance security</strong> with a password-protected system for post editing.</li>
                    </ul>
                </li>
                <li>
                    <p><strong>Edit Post Modal:</strong></p>
                    <ul>
                        <li>Provide a sleek modal for editing posts directly from the frontend.</li>
                        <li><strong>Modify post content, Custom fields, categories, and featured images effortlessly.</strong></li>
                    </ul>
                </li>
            </ol>
            <div class="input-group">
                <a class="button" href="#" target="_blank">Buy Now</a>
            </div>
        </div>
    </div>
    <?php
}

// Add custom column for the Edit Password
function eppb_add_edit_password_column($columns) {
    $columns['edit_password'] = 'Edit Password'; // Add the column for Edit Password
    return $columns;
}
add_filter('manage_posts_columns', 'eppb_add_edit_password_column');

// Show the edit password and copy edit link button in the custom column
function eppb_show_edit_button_link_column($column, $post_id) {
    if ($column === 'edit_password') {
        // Get the edit password and edit code
        $edit_password = get_post_meta($post_id, '_edit_password', true);
        $edit_code = get_post_meta($post_id, '_edit_code', true);

        // Display the edit password
        echo esc_html($edit_password);

        // Generate the edit link
        if ($edit_code) {
            $edit_link = get_permalink($post_id) . '?edit=' . esc_html($edit_code);

            // Add the copy button
            echo ' <button class="copy-edit-link" data-link="' . esc_url($edit_link) . '">Copy Edit Link</button>';
        }
    }
}
add_action('manage_posts_custom_column', 'eppb_show_edit_button_link_column', 10, 2);


// Enqueue jQuery and add inline script to handle the Copy Edit Link button functionality
function eppb_admin_enqueue_scripts() {
    wp_enqueue_script('jquery');

    // Inline script to handle copy to clipboard functionality
    $inline_script = "
        jQuery(document).ready(function ($) {
            $('.copy-edit-link').on('click', function () {
                var link = $(this).data('link');
                navigator.clipboard.writeText(link).then(() => {
                    $(this).text('Link Copied!'); // Change button text on success
                    setTimeout(() => {
                        $(this).text('Copy Edit Link'); // Reset button text after 2 seconds
                    }, 2000);
                }).catch(err => {
                    console.error('Could not copy link: ', err);
                });
            });
        });
    ";

    wp_add_inline_script('jquery', $inline_script);
}
add_action('admin_enqueue_scripts', 'eppb_admin_enqueue_scripts');


// Enqueue scripts and styles for frontend
function eppb_enqueue_scripts() {
    if (is_single()) {
        wp_enqueue_script('jquery');
        wp_enqueue_script('jquery-ui-dialog');
        wp_enqueue_style('eppb-css', plugins_url('/assets/css/style.css', __FILE__), false, '1.0.0', 'all');
        
        // Register and enqueue the main stylesheet in the frontend
        wp_register_style('eppb-main-style', plugins_url('/assets/css/main-style.css', __FILE__));
        wp_enqueue_style('eppb-main-style');

        wp_register_script('eppb-js', plugins_url('/assets/js/plugin-scripts.js', __FILE__), array('jquery'), '1.0.0', true);
        wp_enqueue_script('eppb-js');
    }
}
add_action('wp_enqueue_scripts', 'eppb_enqueue_scripts');

// Enqueue styles for admin area
function eppb_enqueue_admin_styles() {
    // Register and enqueue the main stylesheet in the admin area
    wp_register_style('eppb-main-style', plugins_url('/assets/css/main-style.css', __FILE__));
    wp_enqueue_style('eppb-main-style');
}
add_action('admin_enqueue_scripts', 'eppb_enqueue_admin_styles');

// Add Edit Password field to post editor
function eppb_add_edit_password_field() {
    echo '<label for="edit_password">Edit Password:</label>';
    echo '<input type="password" name="edit_password" id="edit_password" class="edit-password-field" />';
}
add_action('edit_form_after_title', 'eppb_add_edit_password_field');

// Generate and validate the edit password
function eppb_generate_and_validate_edit_password($post_id, $entered_password = '') {
    $stored_password = get_post_meta($post_id, '_edit_password', true);

    if (empty($stored_password)) {
        $new_password = eppb_generate_random_password(8);
        update_post_meta($post_id, '_edit_password', $new_password);
    }

    return ($entered_password === $stored_password);
}
add_action('save_post', 'eppb_generate_and_validate_edit_password');

// Generate a random password
function eppb_generate_random_password($length = 8) {
    $characters = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789';
    return substr(str_shuffle(str_repeat($characters, ceil($length / strlen($characters)))), 0, $length);
}

// Include custom script and edit button on the frontend
function eppb_include_custom_script_and_edit_button() {
    if (is_single()) {
        global $post;
        $post_id = $post->ID;

        eppb_generate_and_validate_edit_password($post_id, '');
        $edit_password = get_post_meta($post_id, '_edit_password', true);
        ?>

    <button class="open-password-modal edit-password-button" data-post-id="<?php echo esc_attr($post_id); ?>" style="display: none;">Edit This Post</button>

        <div id="password-modal-<?php echo esc_attr($post_id); ?>" title="Enter Edit Password" class="password-modal" style="display:none;">
            <form method="post" action="" id="edit-password-form-<?php echo esc_attr($post_id); ?>">
                <?php wp_nonce_field('edit_post_password_' . $post_id); ?>
                <label for="edit_password_<?php echo esc_attr($post_id); ?>">Enter Edit Password:</label>
                <input type="password" name="edit_password_<?php echo esc_attr($post_id); ?>" id="edit_password_<?php echo esc_attr($post_id); ?>" class="edit-password-field" />
                <button type="submit" name="submit_edit_password" value="<?php echo esc_attr($post_id); ?>" class="check-password-button">Check Password</button>
            </form>
        </div>

        <div id="edit-post-modal-<?php echo esc_attr($post_id); ?>" title="Edit Post" class="edit-post-modal" style="display:none;">
            <button id="close-modal-<?php echo esc_attr($post_id); ?>" class="close-modal">Close</button>     
            <form method="post" action="" id="edit-post-form-<?php echo esc_attr($post_id); ?>">
                <?php wp_nonce_field('edit_post_content_' . $post_id); ?>
                <textarea name="post_content" class="edit-post-content-field"><?php echo esc_textarea(get_post_field('post_content', $post_id)); ?></textarea>
                <button type="submit" name="submit_edit_post" value="<?php echo esc_attr($post_id); ?>" class="submit-edit-button">Submit Edit</button>
            </form>
        </div>

        <?php
        if (isset($_POST['submit_edit_password']) && $_POST['submit_edit_password'] == $post_id) {
            if (!isset($_POST['_wpnonce']) || !wp_verify_nonce(sanitize_text_field(wp_unslash($_POST['_wpnonce'])), 'edit_post_password_' . $post_id)) {
                return;
            }

            if (isset($_POST['edit_password_' . $post_id])) {
                $entered_password = sanitize_text_field(wp_unslash($_POST['edit_password_' . $post_id]));
                if (eppb_generate_and_validate_edit_password($post_id, $entered_password)) {
                    $inline_script = "
                        jQuery(document).ready(function ($) {
                            $('#edit-post-modal-" . esc_attr($post_id) . "').dialog({
                                modal: true,
                                width: 500,
                                height: 400,
                                resizable: false,
                                closeOnEscape: true
                            });
                        });
                    ";
                    wp_add_inline_script('eppb-js', $inline_script);
                } else {
                    $inline_script = "
                        jQuery(document).ready(function ($) {
                            alert('Invalid password!');
                        });
                    ";
                    wp_add_inline_script('eppb-js', $inline_script);
                }
            }
        }

        if (isset($_POST['submit_edit_post']) && $_POST['submit_edit_post'] == $post_id) {
            if (!isset($_POST['_wpnonce']) || !wp_verify_nonce(sanitize_text_field(wp_unslash($_POST['_wpnonce'])), 'edit_post_content_' . $post_id)) {
                return;
            }

            if (isset($_POST['post_content'])) {
                $updated_content = sanitize_text_field(wp_unslash($_POST['post_content']));

                // Update the post with new content and set status to pending
                $post_data = array(
                    'ID'           => $post_id,
                    'post_content' => $updated_content,
                    'post_status'  => 'pending',  // Set post to pending review
                );

                wp_update_post($post_data);

                // Add a success message or redirect the user
                $inline_script = "
                    jQuery(document).ready(function ($) {
                        alert('Post submitted for review.');
                    });
                ";
                wp_add_inline_script('eppb-js', $inline_script);
            }
        }
    }
}
add_action('wp_footer', 'eppb_include_custom_script_and_edit_button');
